package views;

import javafx.beans.property.Property;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ColorPicker;
import javafx.scene.control.Label;
import javafx.scene.control.Spinner;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import model.Manager;
import model.cellule.Cellule;
import model.plateau.Plateau;

import java.util.Random;

public class VueJeu {
    @FXML
    private Button random;

    @FXML
    private GridPane map;

    @FXML
    private ColorPicker color;

    @FXML
    private Spinner<Integer> rowGame;

    @FXML
    private Spinner<Integer> colGame;

    @FXML
    private Label nbRowGame;

    @FXML
    private Label nbColGame;

    private Manager manager;
    private Color deathColor;
    private Plateau plateau;

    public void createGrid() {
        /*if (manager.getActualiseurCellule().getArbitre().getPlateau().getLongueur() > 100) {
            manager.getActualiseurCellule().getArbitre().getPlateau().setLongueur(100);
        }
        if(manager.getActualiseurCellule().getArbitre().getPlateau().getLongueur() < 20) {
            manager.getActualiseurCellule().getArbitre().getPlateau().setLongueur(20);
        }
        if (manager.getActualiseurCellule().getArbitre().getPlateau().getLargeur() > 50) {
            manager.getActualiseurCellule().getArbitre().getPlateau().setLargeur(50);
        }
        if (manager.getActualiseurCellule().getArbitre().getPlateau().getLargeur() < 20) {
            manager.getActualiseurCellule().getArbitre().getPlateau().setLargeur(20);
        }*/

        map.getChildren().clear();
        for(int i=0; i < colGame.getValue().intValue(); ++i) {
            for(int j=0; j < rowGame.getValue().intValue(); ++j) {
                Rectangle rect = new Rectangle(15, 15, color.getValue());
                rect.fillProperty().bindBidirectional(manager.getActualiseurCellule().getArbitre().getPlateau().getCell(j, i).activeColorProperty());
                map.add(rect, i, j);
            }
        }
    }

    @FXML
    public void changeColor(MouseEvent mouseEvent) {
        try {
            Node source = (Node)mouseEvent.getSource();
            Integer colIndex = GridPane.getColumnIndex(source);
            Integer rowIndex = GridPane.getRowIndex(source);
            map.add(new Rectangle(15, 15, color.getValue()), colIndex.intValue(), rowIndex.intValue());
        } catch (NullPointerException e){}

    }

    public void generateraRandom() {
        //createGrid(manager.getActualiseurCellule().getArbitre().getPlateau().getLargeur(), manager.getActualiseurCellule().getArbitre().getPlateau().getLongueur());
        resetGrid();
        int largeur = manager.getActualiseurCellule().getArbitre().getPlateau().getLargeur();
        int longueur = manager.getActualiseurCellule().getArbitre().getPlateau().getLongueur();
        for(int i=0; i<20; ++i) {
            Random random = new Random();
            manager.getActualiseurCellule().getArbitre().getPlateau().getCell(random.nextInt(longueur), random.nextInt(largeur)).setAlive(true);
        }
    }

    public void initialize() {
        manager = new Manager();
        deathColor = Color.BLACK;
        rowGame.getValueFactory().valueProperty().bindBidirectional((Property) manager.getActualiseurCellule().getArbitre().getPlateau().largeurProperty());
        colGame.getValueFactory().valueProperty().bindBidirectional((Property) manager.getActualiseurCellule().getArbitre().getPlateau().longueurProperty());
        manager.getActualiseurCellule().getArbitre().getPlateau().longueurProperty().addListener((src)->resetGrid());
        manager.getActualiseurCellule().getArbitre().getPlateau().largeurProperty().addListener((src)->resetGrid());
        color.valueProperty().bindBidirectional(Cellule.livingColorProperty());

        // createGrid();

        nbColGame.setText(colGame.getValue().toString());
        nbRowGame.setText(rowGame.getValue().toString());
    }

    public void startGame(ActionEvent actionEvent) {
        manager.lancerJeu();
    }

    public void resetGrid(){
        manager.getActualiseurCellule().getArbitre().getPlateau().resetGrille();
        createGrid();
    }
}
